@@ -18,7 +18,7 @@ from .to_dense_array import to_dense_array
 from .extract_sparse_array import extract_sparse_array
 from .apply_over_blocks import apply_over_blocks
 from .create_dask_array import create_dask_array
-from .chunk_shape import chunk_shape
+from .chunk_grid import chunk_grid
 from .is_sparse import is_sparse
 from .is_masked import is_masked
 
@@ -78,6 +78,7 @@ class DelayedArray:
     - A method for the
       :py:meth:`~delayedarray.extract_dense_array.extract_dense_array` generic.
     - A method for the :py:meth:`~delayedarray.is_masked.is_masked` generic.
+    - A method for the :py:meth:`~delayedarray.chunk_grid.chunk_grid` generic.
 
     If the seed contains sparse data, it should also implement:
 
@@ -88,11 +89,11 @@ class DelayedArray:
 
     Optionally, a seed class may have:
 
-    - A method for the :py:meth:`~delayedarray.chunk_shape.chunk_shape` generic,
-      if there is some preferred dimension in which to take chunks of the array.
     - A method for the
       :py:meth:`~delayedarray.create_dask_array.create_dask_array` generic,
       if the seed is not already compatible with the **dask** package.
+    - a method for the `wrap()` generic, to create a ``DelayedArray``
+      subclass that is specific to this seed class.
     """
 
     def __init__(self, seed):
@@ -724,21 +725,22 @@ class DelayedArray:
 
         Args:
             axis: 
-                A single integer specifying the axis to perform the calculation.
-                Alternatively a tuple or None, see ``numpy.sum`` for details.
+                A single integer specifying the axis over which to calculate
+                the sum. Alternatively, a tuple (multiple axes) or None (no
+                axes), see :py:func:`~numpy.sum` for details.
 
             dtype:
                 NumPy type for the output array. If None, this is automatically
                 chosen based on the type of the ``DelayedArray``, see
-                ``numpy.sum`` for details.
+                :py:func:`~numpy.sum` for details.
 
             buffer_size:
                 Buffer size in bytes to use for block processing. Larger values
                 generally improve speed at the cost of memory.
 
         Returns:
-            A NumPy array containing the summed values. If ``axis = None``,
-            this will be a NumPy scalar instead.
+            A NumPy array containing the sums. If ``axis = None``, this will be
+            a NumPy scalar instead.
         """
         if hasattr(self._seed, "sum"):
             return self._seed.sum(axis=axis, dtype=dtype)
@@ -760,21 +762,22 @@ class DelayedArray:
 
         Args:
             axis: 
-                A single integer specifying the axis to perform the calculation.
-                Alternatively a tuple or None, see ``numpy.mean`` for details.
+                A single integer specifying the axis over which to calculate
+                the mean. Alternatively, a tuple (multiple axes) or None (no
+                axes), see :py:func:`~numpy.mean` for details.
 
             dtype:
                 NumPy type for the output array. If None, this is automatically
                 chosen based on the type of the ``DelayedArray``, see
-                ``numpy.mean`` for details.
+                :py:func:`~numpy.mean` for details.
 
             buffer_size:
                 Buffer size in bytes to use for block processing. Larger values
                 generally improve speed at the cost of memory.
 
         Returns:
-            A NumPy array containing the mean values. If ``axis = None``,
-            this will be a NumPy scalar instead.
+            A NumPy array containing the means. If ``axis = None``, this will
+            be a NumPy scalar instead.
         """
         if hasattr(self._seed, "mean"):
             return self._seed.mean(axis=axis, dtype=dtype)
@@ -796,13 +799,14 @@ class DelayedArray:
 
         Args:
             axis: 
-                A single integer specifying the axis to perform the calculation.
-                Alternatively a tuple or None, see ``numpy.var`` for details.
+                A single integer specifying the axis over which to calculate
+                the variance. Alternatively, a tuple (multiple axes) or None
+                (no axes), see :py:func:`~numpy.var` for details.
 
             dtype:
                 NumPy type for the output array. If None, this is automatically
                 chosen based on the type of the ``DelayedArray``, see
-                ``numpy.var`` for details.
+                :py:func:`~numpy.var` for details.
 
             ddof:
                 Delta in the degrees of freedom to subtract from the denominator.
@@ -847,10 +851,10 @@ def create_dask_array_DelayedArray(x: DelayedArray):
     return create_dask_array(x._seed)
 
 
-@chunk_shape.register
-def chunk_shape_DelayedArray(x: DelayedArray):
-    """See :py:meth:`~delayedarray.chunk_shape.chunk_shape`."""
-    return chunk_shape(x._seed)
+@chunk_grid.register
+def chunk_grid_DelayedArray(x: DelayedArray):
+    """See :py:meth:`~delayedarray.chunk_grid.chunk_grid`."""
+    return chunk_grid(x._seed)
 
 
 @is_sparse.register
