@@ -2,6 +2,7 @@ from bisect import bisect_left
 from typing import Callable, List, Optional, Sequence, Tuple, Union
 from collections import namedtuple
 import numpy
+import copy
 
 from ._isometric import translate_ufunc_to_op_simple, translate_ufunc_to_op_with_args, ISOMETRIC_OP_WITH_ARGS, _choose_operator, _infer_along_with_args
 from ._subset import _spawn_indices, _getitem_subset_preserves_dimensions, _getitem_subset_discards_dimensions, _repr_subset
@@ -13,6 +14,7 @@ from ._mask import (
     _concatenate_unmasked_ndarrays,
     _concatenate_maybe_masked_ndarrays
 )
+from ._statistics import array_mean, array_var, array_sum, _create_offset_multipliers
 
 __author__ = "ltla"
 __copyright__ = "ltla"
@@ -20,8 +22,9 @@ __license__ = "MIT"
 
 
 class SparseNdarray:
-    """The ``SparseNdarray``, as its name suggests, is a sparse n-dimensional
-    array. It is inspired by the ``SVTArray`` class from the `DelayedArray
+    """
+    The ``SparseNdarray``, as its name suggests, is a sparse n-dimensional
+    array. It is inspired by the ``SVT_Array`` class from the `DelayedArray
     R/Bioconductor package <https://bioconductor.org/packages/DelayedArray>`_.
 
     Internally, the ``SparseNdarray`` is represented as a nested list where
@@ -41,7 +44,9 @@ class SparseNdarray:
     All ``index`` objects in the same ``SparseNdarray`` should have the same
     ``dtype`` (defined by the ``index_dtype`` property). ``value`` may be any
     numeric/boolean :py:class:`~numpy.ndarray` but the ``dtype`` should be
-    consistent across all ``value`` objects in the ``SparseNdarray``.
+    consistent across all ``value`` objects in the ``SparseNdarray``. If the
+    array contains masked values, all ``value`` objects should be a
+    ``MaskedArray``, otherwise they should be regular NumPy arrays.
 
     Any entry of any (nested) list may also be None, indicating that the
     corresponding element of the dimension contains no non-zero values. In
@@ -164,6 +169,15 @@ class SparseNdarray:
         return self._index_dtype
 
 
+    @property
+    def is_masked(self) -> bool:
+        """
+        Returns:
+            Whether the values are masked.
+        """
+        return self._is_masked
+
+
     @property
     def contents(self):
         """Contents of the array. This is intended to be read-only and 
@@ -180,8 +194,9 @@ class SparseNdarray:
 
 
     def __repr__(self) -> str:
-        """Pretty-print this ``SparseNdarray``. This uses
-        :py:meth:`~numpy.array2string` and responds to all of its options.
+        """
+        Pretty-print this ``SparseNdarray``. This uses
+        :py:func:`~numpy.array2string` and responds to all of its options.
 
         Returns:
             String containing a prettified display of the array contents.
@@ -199,9 +214,8 @@ class SparseNdarray:
         """Convert a ``SparseNdarray`` to a NumPy array.
 
         Returns:
-            Array of the same type as :py:attr:`~dtype` and shape as
-            :py:attr:`~shape`. This is guaranteed to be in Fortran storage
-            order and to not be a view on other data.
+            Dense array of the same type as :py:attr:`~dtype` and shape as
+            :py:attr:`~shape`.
         """
         indices = _spawn_indices(self._shape)
         return _extract_dense_array_from_SparseNdarray(self, indices)
@@ -767,6 +781,122 @@ class SparseNdarray:
         return _transpose_SparseNdarray(self, axes)
 
 
+    def sum(self, axis: Optional[Union[int, Tuple[int, ...]]] = None, dtype: Optional[numpy.dtype] = None) -> numpy.ndarray:
+        """
+        Take the sum of values across the ``SparseNdarray``, possibly over a
+        given axis or set of axes.
+
+        Args:
+            axis: 
+                A single integer specifying the axis over which to calculate
+                the sum. Alternatively, a tuple (multiple axes) or None (no
+                axes), see :py:func:`~numpy.sum` for details.
+
+            dtype:
+                NumPy type for the output array. If None, this is automatically
+                chosen based on the type of the ``SparseNdarray``, see
+                :py:func:`~numpy.sum` for details.
+
+        Returns:
+            A NumPy array containing the summed values. If ``axis = None``,
+            this will be a NumPy scalar instead.
+        """
+        return array_sum(
+            self, 
+            axis=axis, 
+            dtype=dtype,
+            reduce_over_x=_reduce_SparseNdarray,
+            masked=self._is_masked,
+        )
+
+
+    def mean(self, axis: Optional[Union[int, Tuple[int, ...]]] = None, dtype: Optional[numpy.dtype] = None) -> numpy.ndarray:
+        """
+        Take the mean of values across the ``SparseNdarray``, possibly over a
+        given axis or set of axes.
+
+        Args:
+            axis: 
+                A single integer specifying the axis over which to calculate
+                the mean. Alternatively, a tuple (multiple axes) or None (no
+                axes), see :py:func:`~numpy.mean` for details.
+
+            dtype:
+                NumPy type for the output array. If None, this is automatically
+                chosen based on the type of the ``SparseNdarray``, see
+                :py:func:`~numpy.mean` for details.
+
+        Returns:
+            A NumPy array containing the mean values. If ``axis = None``,
+            this will be a NumPy scalar instead.
+        """
+        return array_mean(
+            self, 
+            axis=axis, 
+            dtype=dtype,
+            reduce_over_x=_reduce_SparseNdarray,
+            masked=self._is_masked,
+        )
+
+
+    def var(self, axis: Optional[Union[int, Tuple[int, ...]]] = None, dtype: Optional[numpy.dtype] = None, ddof: int = 0) -> numpy.ndarray:
+        """
+        Take the variances of values across the ``SparseNdarray``, possibly
+        over a given axis or set of axes.
+
+        Args:
+            axis: 
+                A single integer specifying the axis over which to calculate
+                the variance. Alternatively, a tuple (multiple axes) or None
+                (no axes), see :py:func:`~numpy.var` for details.
+
+            dtype:
+                NumPy type for the output array. If None, this is automatically
+                chosen based on the type of the ``SparseNdarray``, see
+                :py:func:`~numpy.var` for details.
+
+            ddof:
+                Delta in the degrees of freedom to subtract from the denominator.
+                Typically set to 1 to obtain the sample variance.
+
+        Returns:
+            A NumPy array containing the variances. If ``axis = None``,
+            this will be a NumPy scalar instead.
+        """
+        return array_var(
+            self, 
+            axis=axis, 
+            dtype=dtype,
+            ddof=ddof,
+            reduce_over_x=_reduce_SparseNdarray,
+            masked=self._is_masked,
+        )
+
+
+    # Other stuff
+    def __copy__(self) -> "SparseNdarray":
+        """
+        Returns:
+            A deep copy of this object.
+        """
+        return SparseNdarray(
+            shape=self._shape, 
+            contents=copy.deepcopy(self._contents), 
+            index_dtype=self._index_dtype, 
+            dtype=self._dtype, 
+            is_masked=self._is_masked, 
+            check=False
+        )
+
+
+    def copy(self) -> "SparseNdarray":
+        """
+        Returns:
+            A deep copy of this object.
+        """
+        return self.__copy__()
+
+
 #########################################################
 #########################################################
 
@@ -973,19 +1103,14 @@ def _extract_dense_array_from_SparseNdarray(x: SparseNdarray, subset: Tuple[Sequ
     output = numpy.zeros((*reversed(idims),), dtype=x._dtype)
 
     if x._contents is not None:
-        # Wrapping it in a masked array so that masked values are respected.
-        output = numpy.ma.MaskedArray(output, mask=False)
-
+        if x._is_masked:
+            output = numpy.ma.MaskedArray(output, mask=False)
         ndim = len(x._shape)
         if ndim > 1:
             _recursive_extract_dense_array(x._contents, subset, subset_summary=subset_summary, output=output, dim=ndim-1)
         else:
             _extract_sparse_vector_to_dense(x._contents[0], x._contents[1], subset_summary=subset_summary, output=output)
 
-        # Stripping out the mask if there were, in fact, no masked values.
-        if isinstance(output.mask, bool) and not output.mask:
-            output = output.data
-
     return output.T
 
 
@@ -1090,7 +1215,7 @@ def _extract_sparse_array_from_SparseNdarray(x: SparseNdarray, subset: Tuple[Seq
         else:
             new_contents = _extract_sparse_vector_to_sparse(x._contents[0], x._contents[1], subset_summary)
 
-    return SparseNdarray(shape=(*idims,), contents=new_contents, index_dtype=x.index_dtype, dtype=x.dtype, check=False)
+    return SparseNdarray(shape=(*idims,), contents=new_contents, index_dtype=x.index_dtype, dtype=x.dtype, is_masked=x.is_masked, check=False)
 
 
 #########################################################
@@ -1150,7 +1275,7 @@ def _transform_sparse_array_from_SparseNdarray(x: SparseNdarray, f: Callable, ou
         else:
             new_contents = _transform_sparse_vector((), indices=x._contents[0], values=x._contents[1], payload=payload)
 
-    return SparseNdarray(shape=x._shape, contents=new_contents, index_dtype=x.index_dtype, dtype=output_dtype, check=False)
+    return SparseNdarray(shape=x._shape, contents=new_contents, index_dtype=x.index_dtype, dtype=output_dtype, is_masked=x.is_masked, check=False)
 
 
 #########################################################
@@ -1276,7 +1401,7 @@ def _binary_operation_on_SparseNdarray(x: SparseNdarray, y: SparseNdarray, opera
         else:
             new_contents = _binary_operate_sparse_vector(x._contents, y._contents, payload=payload)
 
-    return SparseNdarray(shape=x._shape, contents=new_contents, index_dtype=x.index_dtype, dtype=dummy.dtype, check=False)
+    return SparseNdarray(shape=x._shape, contents=new_contents, index_dtype=x.index_dtype, dtype=dummy.dtype, is_masked=x.is_masked, check=False)
 
 
 #########################################################
@@ -1430,7 +1555,7 @@ def _transpose_SparseNdarray(x: SparseNdarray, perm):
             dim=ndim - 1,
         )
 
-    return SparseNdarray(shape=(*new_shape,), contents=new_contents, index_dtype=x._index_dtype, dtype=x._dtype, check=False)
+    return SparseNdarray(shape=(*new_shape,), contents=new_contents, index_dtype=x._index_dtype, dtype=x._dtype, is_masked=x.is_masked, check=False)
 
 
 #########################################################
@@ -1532,6 +1657,7 @@ def _concatenate_SparseNdarrays(xs: List[SparseNdarray], along: int):
 
     output_dtype = _concatenate_unmasked_ndarrays(dummy_collected, axis=0).dtype
     output_index_dtype = _concatenate_unmasked_ndarrays(dummy_collected_index, axis=0).dtype
+    output_is_masked = any(y._is_masked for y in xs)
 
     all_none = True
     for con in all_contents:
@@ -1541,7 +1667,6 @@ def _concatenate_SparseNdarrays(xs: List[SparseNdarray], along: int):
     new_contents = None
     if not all_none:
         offset = None
-        index_dtype = output_index_dtype
         ndim = len(new_shape)
 
         # Upgrading the index type to ensure we can hold the output, if we're
@@ -1554,17 +1679,17 @@ def _concatenate_SparseNdarrays(xs: List[SparseNdarray], along: int):
                 offset.append(last)
                 last += shape[along]
 
-            for candidate in [index_dtype, numpy.uint32, numpy.uint64]:
+            for candidate in [output_index_dtype, numpy.uint32, numpy.uint64]:
                 if last < numpy.iinfo(candidate).max:
-                    index_dtype = candidate
+                    output_index_dtype = candidate
                     break
 
         payload = _ConcatenatePayload(
             shapes=all_shapes, 
             offset=offset, 
             output_dtype=output_dtype, 
-            output_index_dtype=index_dtype, 
-            output_is_masked=any(y._is_masked for y in xs)
+            output_index_dtype=output_index_dtype, 
+            output_is_masked=output_is_masked
         )
 
         if ndim > 1:
@@ -1578,4 +1703,43 @@ def _concatenate_SparseNdarrays(xs: List[SparseNdarray], along: int):
                     outval.append(c[1])
             new_contents = _concatenate_sparse_vectors(outidx, outval, payload)
 
-    return SparseNdarray(shape=(*new_shape,), contents=new_contents, dtype=output_dtype, index_dtype=output_index_dtype, check=False)
+    return SparseNdarray(shape=(*new_shape,), contents=new_contents, dtype=output_dtype, index_dtype=output_index_dtype, is_masked=output_is_masked, check=False)
+
+
+#########################################################
+#########################################################
+
+
+_ReductionPayload = namedtuple("_ReductionPayload", [ "multipliers", "operation" ])
+
+
+def _reduce_sparse_vector(idx: numpy.ndarray, val: numpy.ndarray, payload: _ReductionPayload, offset: int = 0):
+    for j, ix in enumerate(idx):
+        extra = payload.multipliers[0] * ix
+        payload.operation(offset + extra, val[j])
+    return
+
+
+def _recursive_reduce_SparseNdarray(contents, payload: _ReductionPayload, dim: int, offset: int  = 0):
+    mult = payload.multipliers[dim]
+    if dim == 1:
+        for i, con in enumerate(contents):
+            if con is not None:
+                _reduce_sparse_vector(con[0], con[1], payload, offset = offset + mult * i)
+    else:
+        for i, con in enumerate(contents):
+            if con is not None:
+                _recursive_reduce_SparseNdarray(con, payload, dim = dim - 1, offset = offset + mult * i)
+    return
+
+
+def _reduce_SparseNdarray(x: SparseNdarray, axes: List[int], operation: Callable):
+    if x.contents is not None:
+        multipliers = _create_offset_multipliers(x.shape, axes)
+        payload = _ReductionPayload(operation=operation, multipliers=multipliers)
+        ndim = len(x.shape)
+        if ndim == 1:
+            _reduce_sparse_vector(x.contents[0], x.contents[1], payload)
+        else:
+            _recursive_reduce_SparseNdarray(x.contents, payload, dim=ndim - 1)
+    return        
